import pandas as pd
from pandas import DataFrame
from pathlib import Path
from os import path
import numpy as np
import torch
from sklearn.preprocessing import robust_scale

workspace_path = Path(path.abspath(path.dirname(__file__))) / ".."
data_path = workspace_path / "data"


def load_data(filename):
    data: DataFrame = pd.read_csv(
        data_path / filename,
        header=None,
        names=[
            "dureation",
            "protocol",
            "service",
            "flag",
            "src_bytes",
            "dst_bytes",
            "land",
            "wrong_fragment",
            "urgent",
            "hot",
            "num_failed_logins",
            "logged_in",
            "num_compromised",
            "root_shell",
            "su_attempted",
            "num_root",
            "num_file_creations",
            "num_shells",
            "num_access_files",
            "num_outbound_cmds",
            "is_host_login",
            "is_guest_login",
            "count",
            "srv_count",
            "serror_rate",
            "srv_serror_rate",
            "rerror_rate",
            "srv_rerror_rate",
            "same_srv_rate",
            "diff_srv_rate",
            "srv_diff_host_rate",
            "dst_host_count",
            "dst_host_srv_count",
            "dst_host_same_srv_rate",
            "dst_host_diff_srv_rate",
            "dst_host_same_src_port_rate",
            "dst_host_srv_diff_host_rate",
            "dst_host_serror_rate",
            "dst_host_srv_serror_rate",
            "dst_host_rerror_rate",
            "dst_host_srv_rerror_rate",
            "class",
            "level",
        ],
        dtype={1: "category", 2: "category", 3: "category", 41: "category"},
    )

    # data = data.rename(columns={41: "class", 42: "level"})

    data["protocol"] = data["protocol"].cat.codes
    data["service"] = data["service"].cat.codes
    data["flag"] = data["flag"].cat.codes
    return data


def prepare_data(data, sequence=False, drop_label=True):
    label_cat = data["class"].cat.categories
    normal_value = np.where(label_cat == "normal")[0]

    # converting the `normal` class to 0 and others to 1
    lables = (data["class"].cat.codes.to_numpy() != normal_value).astype(int)
    levels = data["level"].to_numpy()

    # drop the label column
    if drop_label:
        data = data.drop(columns=["class", "level"])

    x = data.to_numpy()

    # min-max normalization
    x = robust_scale(x)
    x = torch.from_numpy(x)

    if sequence:
        x = x.unsqueeze(1)

    return x, torch.from_numpy(lables), torch.from_numpy(levels)
