import classes from "./App.module.scss";
import { useState, useEffect, useRef } from "react";
import Graph from "./Graph";

async function toggleTraffic(paused) {
  const path = paused ? "pause" : "resume";

  await fetch(`/control/${path}`, {
    method: "POST",
  });
}

function App() {
  const [traffic, setTraffic] = useState([]);
  const [trafficPaused, setTrafficPaused] = useState(true);
  const [network, setNetwork] = useState([]);
  const [bufferWindow, setBufferWindow] = useState(10);

  const ws = useRef(null);

  // fetch the network data
  useEffect(() => {
    async function fetchNetwork() {
      const res = await fetch("/network");

      setNetwork(await res.json());
    }

    fetchNetwork();
  }, []);

  useEffect(() => {
    const _ws = new WebSocket("ws://localhost:3010/traffic");

    ws.current = _ws;

    return () => _ws.close();
  }, []);

  useEffect(() => {
    if (ws.current)
      ws.current.onmessage = ({ data }) => {
        const traffic = JSON.parse(data);
        const now = Date.now();
        traffic.received = now;
        setTraffic((s) =>
          s
            .filter((n) => now - n.received < Math.max(bufferWindow, 1) * 1000)
            .concat([traffic])
        );
      };
  }, [bufferWindow]);

  useEffect(() => {
    toggleTraffic(trafficPaused);
  }, [trafficPaused]);

  useEffect(() => {
    document.addEventListener("visibilitychange", () => {
      if (document.hidden) {
        setTrafficPaused(true);
      }
    });
  });

  return (
    <div className={classes.app}>
      <div className={classes.toolbar}>
        <button
          onClick={(e) => {
            setTrafficPaused((s) => !s);
          }}
        >
          {trafficPaused ? "Start" : "Pause"}
        </button>
        <input
          type="number"
          onChange={(e) => setBufferWindow(e.target.value)}
          value={bufferWindow}
        ></input>
      </div>
      <div className={classes.graph}>
        <Graph name="prediction" network={network} traffic={traffic}></Graph>
        <Graph
          name="truth"
          network={network}
          traffic={traffic.map((t) => ({
            ...t,
            prediction: { class: t.class, confidence: 1 },
          }))}
        ></Graph>
      </div>
    </div>
  );
}

export default App;
