import { useEffect, useRef, useState } from "react";
import cytoscape from "cytoscape";
import fcose from "cytoscape-fcose";
import classes from "./Graph.module.scss";
import { interpolate } from "d3-interpolate";

fcose(cytoscape);

const intercolor = interpolate("yellow", "red");

export default function Graph({ name, network, traffic, onTrafficSelect }) {
  const [drillTraffic, setDrillTraffic] = useState(null);

  const graphRef = useRef();
  const cy = useRef();
  // render the graph
  useEffect(() => {
    if (graphRef.current && network.length > 0) {
      cy.current = cytoscape({
        container: graphRef.current,
        ready() {
          this.nodes().forEach((node) => {
            let size = 5;
            node.css("width", size);
            node.css("height", size);
          });

          let pin = false;

          // this.edges().on("mouseover", (e) => {
          //   console.log("hover", e.target.data("traffic"));
          // });
          this.edges()
            .on("mouseover", (e) => {
              if (!pin) {
                setDrillTraffic(e.target.data("traffic"));
                onTrafficSelect?.(e.target.data("traffic"));
              }
              // console.log("hover", e.target.data("traffic"));
            })
            .on("mouseout", (e) => {
              if (!pin) {
                setDrillTraffic(null);
                onTrafficSelect?.(null);
              }
            });

          this.edges().on("click", (e) => {
            e.stopPropagation();
            setDrillTraffic(e.target.data("traffic"));
            onTrafficSelect?.(e.target.data("traffic"));
            e.target.data("pin", true);
            pin = true;
          });

          this.on("click", (e) => {
            pin = false;
            setDrillTraffic(null);
            onTrafficSelect?.(null);
            this.edges().data("pin", false);
          });

          this.layout({ name: "cose", animate: false }).run();
          this.nodes().lock();
        },
        elements: JSON.parse(JSON.stringify(network)),
        style: [
          // the stylesheet for the graph
          {
            selector: "node",
            style: {
              "background-color": "#444",
            },
          },

          {
            selector: "edge",
            style: {
              width: 1,
              "line-color": "#222",
              "target-arrow-color": "#222",
              "target-arrow-shape": "vee",
              "curve-style": "bezier",
            },
          },
        ],
      });
    }
  }, [network, onTrafficSelect]);

  // update the traffic
  useEffect(() => {
    if (cy.current) {
      // cy.current.edges().style({
      //   "line-color": "#ccc",
      //   "target-arrow-color": "#ccc",
      // });

      const idMap = {};
      for (const t of traffic) {
        idMap[`${t.src}-${t.dst}`] = t;
      }

      cy.current.edges().forEach((edge) => {
        const t = idMap[edge.data("id")];

        if (edge.data("traffic") && !t) {
          edge.data("traffic", null).animate({
            style: {
              width: 1,
              "line-color": "#222",
              "target-arrow-color": "#222",
            },
          });
        } else if (t && !edge.data("traffic")) {
          const color =
            t.prediction.class === "normal"
              ? "green"
              : intercolor(t.prediction.confidence);

          edge.data("traffic", t).animate({
            style: {
              width: 2,
              "line-color": color,
              "target-arrow-color": color,
            },
          });
        }
      });
    }
  }, [traffic]);

  return (
    <div className={classes.container}>
      <h1 className={classes.title}>{name}</h1>
      <div className={classes.graph} ref={graphRef}></div>
      <div className={classes.sidebar}>
        {drillTraffic && (
          <>
            <span>
              {drillTraffic.prediction.class}: (
              {drillTraffic.prediction.confidence}) | {drillTraffic.class}
            </span>
            <span>Protocol: {drillTraffic.protocol}</span>
            <span>Service: {drillTraffic.service}</span>
            <span>Source bytes: {drillTraffic.src_bytes}</span>
            <span>Destination bytes: {drillTraffic.dst_bytes}</span>
            <span>Compromised: {drillTraffic.num_compromised}</span>
            <span>Login Fails: {drillTraffic.num_failed_logins}</span>
            <span>Root: {drillTraffic.num_root}</span>
            <span>Shells: {drillTraffic.num_shells}</span>
          </>
        )}
      </div>
    </div>
  );
}
